import java.awt.*;
import java.util.*;
import javax.swing.*;

public class CheckerBoard extends Board {

  class Selection {
    public int x, y;
  
    public Selection(int x, int y) {
      this.x = x;
      this.y = y;
    }
    
  }
 
  private int FIELD_SIZE = 30;
  private int X_OFFSET = 30;
  private int Y_OFFSET = 20;
  
  private GameClient owner;
  private String stateStr;
  private char state[][];
  private Vector selection;
  private char who;

  private ImageIcon blackC, blackK, whiteC, whiteK;

  public CheckerBoard(GameClient owner) {
    this.owner = owner;
    stateStr = "";
    state = new char[8][8];
    selection = new Vector();
    initComponents();
    blackC = new ImageIcon("checkers/black_c.gif");
    blackK = new ImageIcon("checkers/black_k.gif");
    whiteC = new ImageIcon("checkers/white_c.gif");
    whiteK = new ImageIcon("checkers/white_k.gif");
    who = '_';
  }

  private void initComponents() {//GEN-BEGIN:initComponents
    addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(java.awt.event.MouseEvent evt) {
        mouseClicker(evt);
      }
    });
  }

  private void mouseClicker(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseClicker
    boolean finished = false;
    int x = evt.getX() - X_OFFSET;
    int y =  evt.getY() - Y_OFFSET;
    if ((x < 0) || (y < 0) || (x > FIELD_SIZE * 8) || (y > FIELD_SIZE * 8))
      return;
    x /= FIELD_SIZE;
    y /= FIELD_SIZE;
    if (selection == null)
      selection = new Vector();
    for (int i = 0; i < selection.size(); i++) {
      Selection s = (Selection)selection.elementAt(i);
      if ((s.x == x) && (s.y == y)) {
        if (i == selection.size() - 1) {
          selection.removeElementAt(selection.size() - 1);
          finished = true;
          break;
        }
        else
          return;
      }
    }
    if ((!finished) && (selection.size() < 4))
      selection.addElement(new Selection(x, y));
    this.repaint(X_OFFSET + x * FIELD_SIZE - 1, Y_OFFSET + y * FIELD_SIZE - 1,
                 FIELD_SIZE + 2, FIELD_SIZE + 2);
    owner.setMove(selectionToString());
  }

  private String selectionToString() {
    String result = "";
    if (selection != null)
      for (int i = 0; i < selection.size(); i++) {
        Selection s = (Selection)selection.elementAt(i);
        result += "" + (char)('a' + (char)s.x) + "" + (char)('1' + (char)(7 - s.y));
      }
    return result;
  }
    
  public void drawEmptyBoard(Graphics g) {
    int x, y, width;
    char c;
    Color white = new Color(240, 240, 200);
    Color black = new Color(120, 120, 100);
    FontMetrics fm = g.getFontMetrics();
    g.setColor(Color.white);
    for (x = 0; x < 8; x++) {
      c = (char)('A' + x);
      width = fm.charWidth(c);
      y = X_OFFSET + x * FIELD_SIZE + FIELD_SIZE/2 - width/2;
      g.drawString("" + c, y, Y_OFFSET - 6);
    }
    for (y = 0; y < 8; y++) {
      c = (char)('8' - y);
      width = fm.charWidth(c);
      x = X_OFFSET - width - 6;
      g.drawString("" + c, x, Y_OFFSET + (y + 1) * FIELD_SIZE - 9);
    }
    for (x = 0; x < 8; x++)
      for (y = 0; y < 8; y++) {
        if ((x + y) % 2 == 0)
          g.setColor(white);
        else
          g.setColor(black);
        g.fillRect(X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE,
                   FIELD_SIZE, FIELD_SIZE);
      }
    g.setColor(Color.white);
    if (who == 'w')
      g.drawString("You have the white pieces.", X_OFFSET + FIELD_SIZE, 276);
    else if (who == 'b')
      g.drawString("You have the black pieces.", X_OFFSET + FIELD_SIZE, 276);
  }
    
  public void paint(Graphics g) {
    int x, y, cy;
    Color lineColor, fillColor;
    drawEmptyBoard(g);
    if (selection != null)
      for (x = 0; x < selection.size(); x++) {
        Selection s = (Selection)selection.elementAt(x);
        switch (x) {
          case 0: g.setColor(new Color(240, 80, 0)); break;
          case 1: g.setColor(new Color(80, 240, 80)); break;
          case 2: g.setColor(new Color(80, 80, 240)); break;
          default: g.setColor(new Color(255, 255, 0)); break;
        }
        g.fillRect(X_OFFSET + s.x * FIELD_SIZE, Y_OFFSET + s.y * FIELD_SIZE,
                   FIELD_SIZE, FIELD_SIZE);
      }
    lineColor = fillColor = null;
    if ((stateStr != null) && (stateStr.length() > 0)) {
      for (x = 0; x < 8; x++)
        for (y = 0; y < 8; y++) {
          switch (state[x][7 - y]) {
            case 'w': whiteC.paintIcon(this, g, X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE); break;
            case 'W': whiteK.paintIcon(this, g, X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE); break;
            case 'b': blackC.paintIcon(this, g, X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE); break;
            case 'B': blackK.paintIcon(this, g, X_OFFSET + x * FIELD_SIZE, Y_OFFSET + y * FIELD_SIZE); break;
          }
        }
    }
  }
  
  public void setBoardState(String aState) {
    stateStr = aState;
    if ((stateStr != null) && (stateStr.length() == 64)) {
      for (int x = 0; x < 8; x++)
        for (int y = 0; y < 8; y++)
          state[x][y] = stateStr.charAt(x + 8 * y);
    }
    selection = new Vector();
    repaint();
  }

  public void setTimeOut(long ms) {
  }

  public void setWho(char who) {
    this.who = who;
  }

}
