import java.io.*;
import java.net.*;

public class Connection extends Thread implements Comparable {
  private final int MAX_INACTIVE_TIME = 15000;
     // check every 15 seconds if the client is still alive
  private Socket socket;
  private InputStream inStream;
  private OutputStream outStream;
  private boolean terminated;
  private ConnectionMaster owner;
  private Cryptor localCryptor, remoteCryptor;
  private String inBuffer;
  private byte byteBuffer[];
  private long lastActive;

  public Connection(ConnectionMaster anOwner, Socket s) {
    socket = s;
    owner = anOwner;
    localCryptor = new RSA();
    remoteCryptor = null;
    inBuffer = "";
    byteBuffer = new byte[2048];
    terminated = false;
    lastActive = System.currentTimeMillis();
    try {
      inStream = s.getInputStream();
      outStream = s.getOutputStream();
    }
    catch (IOException ioe) {
      terminated = true;
      s = null;
    }
  }
  
  public void run() {
    int available = 0;
    while (!terminated) try {
      sleep(40);
      available = inStream.available();
      if (available > 2000)
        available = 2000;
      if (available > 0) {
        lastActive = System.currentTimeMillis();
        inStream.read(byteBuffer, 0, available);
        inBuffer += new String(byteBuffer, 0, available);
        available = inBuffer.indexOf("\n");
        if (available > 0) {
          String message = inBuffer.substring(0, available);
          inBuffer = inBuffer.substring(available + 1);
          if (remoteCryptor == null) {
//System.out.println("Message to process: " + message);
            if (message.compareTo("getkey") == 0)
              sendMessage(Msg.messageString(Msg.OK_PUBKEY) +
                          localCryptor.exportPublicKey());
            if ((message.length() > 7) &&
                (message.substring(0, 7).compareTo("setkey ") == 0)) {
              message = message.substring(7);
              if ((message.length() > 3) &&
                  (message.substring(0, 3).compareTo("RSA") == 0))
                remoteCryptor = new RSA(message);
              if ((remoteCryptor != null) && (remoteCryptor.getPublicKey() != null))
                sendMessage(Msg.messageString(Msg.OK_PUBKEY_RECVD));
              else {
                remoteCryptor = null;
                sendMessage(Msg.messageString(Msg.ERR_ILLEGAL_FORMAT));
              }
            }
          }
          else {
            message = remoteCryptor.verify(localCryptor.decrypt(message));
//System.out.println("Message to process: " + message);
            if (message != null)
              owner.processMessage(this, message);
            else
              sendMessage(Msg.messageString(Msg.ERR_ILLEGAL_FORMAT));
          }
        }
      }
      else
        if (System.currentTimeMillis() > lastActive + MAX_INACTIVE_TIME) {
          lastActive = System.currentTimeMillis();
          sendPing();
        }
    }
    catch (Exception e) {
      System.out.println("Mist: " + e);
      e.printStackTrace();
      System.exit(0);
    }
    // terminated == true
    System.out.println("Connection closed.");
    try {
      socket.shutdownInput();
      socket.shutdownOutput();
    } catch (IOException ioe) {
    }
    owner.removeConnection(this);
    socket = null;
  }

  public void sendPing() {
    try {
      outStream.write(new String("ping\n").getBytes());
      outStream.flush();
    } catch (IOException ioe) {
      terminated = true;
    }
  }

  public void sendMessage(String message) {
    if (remoteCryptor != null)
      message = remoteCryptor.encrypt(localCryptor.sign(message));
    message += "\n";
    try {
      outStream.write(message.getBytes());
      outStream.flush();
    } catch (IOException ioe) {
//System.out.println(ioe);
      terminated = true;
    }
  }

  public String toString() {
    return localCryptor.exportPublicKey();
  }

  public int compareTo(Object o) {
    return toString().compareTo(o.toString());
  }

}
