/********************************************************************************
 * Cryptor.java
 * Copyright (c) 2002 by Stefan Buettcher
 *
 * All communication is done encrypted. This means we need to define an
 * interface for this. We will work with public-key encryption. ElGamal and
 * RSA are supported.
 *
 *******************************************************************************/

import java.math.*;
import java.util.*;

public interface Cryptor {
  public BigInteger getModulus();
    // returns the modulus n=pq (RSA) or p (ElGamal)
  public BigInteger getPublicKey();
    // returns the Cryptor's public key

  public String encrypt(String plainText);
    // encrypts a text using the public key
  public String decrypt(String cipherText);
    // tries to decrypt a encrypted text using the private key. returns null upon failure.
  public String sign(String plainText);
    // signs a text using the private key
  public String verify(String plainText);
    // returns null upon failure, the original text if the signature is valid

  public String exportPublicKey();
    // transforms the modulus and the private key into a readable string form

  public BigInteger getRandomPrime();
  public BigInteger getRandomComposed();
    // these are obvious, I think
}
