import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.sound.sampled.*;

public class GameClient extends javax.swing.JFrame {
  private boolean connected, success, canStartValue;
  private Socket connection;
  private OutputStream out;
  private InputStream in;
  private Cryptor localCryptor, remoteCryptor;
  private UpdateThread updater;
  private NewsAgent newsAgent;
  private SoundAgent soundAgent;
  private String gameType, gameName, playerName;
  private Board boardState;
  private int moveCount;

  public boolean isConnected() {
    return connected;
  }
  
  /** Creates new form GameClient */
    public GameClient() {
      connected = false;
      localCryptor = new RSA();
      remoteCryptor = null;
      initComponents();
      boardState = null;
      setupBoard("");
      updater = new UpdateThread(this);
      updater.start();
      newsAgent = new NewsAgent(this, updater);
      newsAgent.start();
      soundAgent = new SoundAgent();
      try {
        URL url = new URL("http", "stefan.buettcher.org", "/cs/gameserver/getserver.php");
        InputStream i = url.openStream();
        Thread.sleep(100);
        String result = "";
        while (i.available() > 0) {
          int len = i.available();
          byte bytes[] = new byte[len];
          i.read(bytes, 0, len);
          result += new String(bytes);
        }
        int index = result.indexOf(',');
        if (index > 0)
          serverEdit.setText(result.substring(0, index));
      }
      catch (Exception e) {
      }
      soundAgent.start();
      soundAgent.playFile("sound/welcome.wav");
    }
    
    private void setupBoard(String gameType) {
      if (boardState != null)
        gamePanel.remove(boardState);
      if (gameType.compareTo("Checkers") == 0)
        boardState = new CheckerBoard(this);
      else if (gameType.compareTo("Chess") == 0)
        boardState = new ChessBoard(this);
      else if (gameType.compareTo("4Gewinnt") == 0)
        boardState = new VierGewinntBoard(this);
      else
        boardState = new EmptyBoard(this);
      gamePanel.add(boardState);
      boardState.setLocation(210, 20);
      boardState.setSize(280, 280);
    }

    private void initComponents() {//GEN-BEGIN:initComponents
      jPanel3 = new javax.swing.JPanel();
      jPanel4 = new javax.swing.JPanel();
      jLabel3 = new javax.swing.JLabel();
      jLabel4 = new javax.swing.JLabel();
      portEdit = new javax.swing.JTextField();
      connectionState = new javax.swing.JLabel();
      connectButton = new javax.swing.JButton();
      jLabel6 = new javax.swing.JLabel();
      playerEdit = new javax.swing.JTextField();
      serverEdit = new javax.swing.JTextField();
      gamePanel = new javax.swing.JPanel();
      canStart = new javax.swing.JCheckBox();
      jButton1 = new javax.swing.JButton();
      moveEdit = new javax.swing.JTextField();
      moveButton = new javax.swing.JButton();
      jLabel5 = new javax.swing.JLabel();
      jLabel8 = new javax.swing.JLabel();
      jLabel9 = new javax.swing.JLabel();
      jLabel10 = new javax.swing.JLabel();
      jLabel11 = new javax.swing.JLabel();
      moveHistory = new java.awt.List();
      jPanel6 = new javax.swing.JPanel();
      gameTypeList = new java.awt.List();
      jPanel7 = new javax.swing.JPanel();
      gameList = new java.awt.List();
      jPanel8 = new javax.swing.JPanel();
      playerList = new java.awt.List();
      jLabel7 = new javax.swing.JLabel();
      messageLabel = new javax.swing.JLabel();
      gameNameEdit = new javax.swing.JTextField();
      createButton = new javax.swing.JButton();
      jSeparator1 = new javax.swing.JSeparator();
      jLabel1 = new javax.swing.JLabel();
      jLabel2 = new javax.swing.JLabel();
      chatEdit = new javax.swing.JTextField();
      sendButton = new javax.swing.JButton();
      chatHistory = new java.awt.List();
      
      addWindowListener(new java.awt.event.WindowAdapter() {
        public void windowClosing(java.awt.event.WindowEvent evt) {
          exitForm(evt);
        }
      });
      
      jPanel3.setLayout(null);
      
      jPanel3.setBackground(new java.awt.Color(0, 60, 120));
      jPanel4.setLayout(null);
      
      jPanel4.setBorder(new javax.swing.border.TitledBorder(null, "Connection", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), java.awt.Color.white));
      jPanel4.setBackground(new java.awt.Color(0, 80, 160));
      jLabel3.setText("Server name/address:");
      jLabel3.setForeground(java.awt.Color.white);
      jLabel3.setBackground(java.awt.Color.white);
      jPanel4.add(jLabel3);
      jLabel3.setBounds(10, 20, 170, 13);
      
      jLabel4.setText("Remote port number:");
      jLabel4.setForeground(java.awt.Color.white);
      jPanel4.add(jLabel4);
      jLabel4.setBounds(10, 40, 170, 13);
      
      portEdit.setText("1234");
      jPanel4.add(portEdit);
      portEdit.setBounds(190, 38, 140, 17);
      
      connectionState.setText("(unconnected)");
      connectionState.setForeground(java.awt.Color.white);
      connectionState.setFont(new java.awt.Font("Dialog", 0, 12));
      jPanel4.add(connectionState);
      connectionState.setBounds(50, 86, 89, 13);
      
      connectButton.setForeground(java.awt.Color.white);
      connectButton.setText("Connect");
      connectButton.setBackground(new java.awt.Color(0, 80, 160));
      connectButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          connectButtonActionPerformed(evt);
        }
      });
      
      jPanel4.add(connectButton);
      connectButton.setBounds(190, 80, 140, 23);
      
      jLabel6.setText("Player name:");
      jLabel6.setForeground(java.awt.Color.white);
      jPanel4.add(jLabel6);
      jLabel6.setBounds(10, 60, 120, 13);
      
      playerEdit.setText("Stu");
      jPanel4.add(playerEdit);
      playerEdit.setBounds(190, 58, 140, 17);
      
      serverEdit.setText("localhost");
      jPanel4.add(serverEdit);
      serverEdit.setBounds(190, 18, 140, 17);
      
      jPanel3.add(jPanel4);
      jPanel4.setBounds(10, 30, 340, 110);
      
      gamePanel.setLayout(null);
      
      gamePanel.setBorder(new javax.swing.border.TitledBorder(null, "Current game", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), java.awt.Color.white));
      gamePanel.setBackground(new java.awt.Color(0, 80, 160));
      canStart.setForeground(java.awt.Color.white);
      canStart.setText("I am ready.");
      canStart.setBackground(new java.awt.Color(0, 80, 160));
      canStart.setBackground(gamePanel.getBackground());
      canStart.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          canStartActionPerformed(evt);
        }
      });
      
      gamePanel.add(canStart);
      canStart.setBounds(10, 20, 150, 21);
      
      jButton1.setForeground(java.awt.Color.white);
      jButton1.setText("Quit game");
      jButton1.setBackground(new java.awt.Color(0, 80, 160));
      jButton1.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          jButton1ActionPerformed(evt);
        }
      });
      
      gamePanel.add(jButton1);
      jButton1.setBounds(10, 276, 180, 23);
      
      gamePanel.add(moveEdit);
      moveEdit.setBounds(10, 124, 70, 17);
      
      moveButton.setForeground(java.awt.Color.white);
      moveButton.setText("Do move");
      moveButton.setBackground(new java.awt.Color(0, 80, 160));
      moveButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          moveButtonActionPerformed(evt);
        }
      });
      
      gamePanel.add(moveButton);
      moveButton.setBounds(90, 121, 100, 23);
      
      jLabel5.setText("Please click here when you");
      jLabel5.setForeground(java.awt.Color.white);
      jLabel5.setFont(new java.awt.Font("Dialog", 0, 12));
      gamePanel.add(jLabel5);
      jLabel5.setBounds(26, 40, 168, 13);
      
      jLabel8.setText("are ready to start playing.");
      jLabel8.setForeground(java.awt.Color.white);
      jLabel8.setFont(new java.awt.Font("Dialog", 0, 12));
      gamePanel.add(jLabel8);
      jLabel8.setBounds(26, 54, 161, 13);
      
      jLabel9.setText("Please enter your move here");
      jLabel9.setForeground(java.awt.Color.white);
      jLabel9.setFont(new java.awt.Font("Dialog", 0, 12));
      gamePanel.add(jLabel9);
      jLabel9.setBounds(10, 86, 176, 13);
      
      jLabel10.setText("(or click on the board).");
      jLabel10.setForeground(java.awt.Color.white);
      jLabel10.setFont(new java.awt.Font("Dialog", 0, 12));
      gamePanel.add(jLabel10);
      jLabel10.setBounds(10, 100, 143, 13);
      
      jLabel11.setText("Move history");
      jLabel11.setForeground(java.awt.Color.white);
      gamePanel.add(jLabel11);
      jLabel11.setBounds(10, 170, 93, 13);
      
      moveHistory.setBackground(java.awt.Color.white);
      gamePanel.add(moveHistory);
      moveHistory.setBounds(10, 188, 180, 70);
      
      jPanel3.add(gamePanel);
      gamePanel.setBounds(10, 150, 510, 310);
      
      jPanel6.setLayout(null);
      
      jPanel6.setBorder(new javax.swing.border.TitledBorder(null, "Available game types", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), java.awt.Color.white));
      jPanel6.setBackground(new java.awt.Color(0, 80, 160));
      gameTypeList.setFont(new java.awt.Font("Dialog", 1, 12));
      gameTypeList.setEnabled(false);
      gameTypeList.setBackground(java.awt.Color.white);
      gameTypeList.addMouseListener(new java.awt.event.MouseAdapter() {
        public void mouseClicked(java.awt.event.MouseEvent evt) {
          gameTypeListMouseClicked(evt);
        }
      });
      
      jPanel6.add(gameTypeList);
      gameTypeList.setBounds(10, 20, 140, 80);
      
      jPanel3.add(jPanel6);
      jPanel6.setBounds(360, 30, 160, 110);
      
      jPanel7.setLayout(null);
      
      jPanel7.setBorder(new javax.swing.border.TitledBorder(null, "Available games", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), java.awt.Color.white));
      jPanel7.setBackground(new java.awt.Color(0, 80, 160));
      gameList.setFont(new java.awt.Font("Dialog", 1, 12));
      gameList.setEnabled(false);
      gameList.setBackground(java.awt.Color.white);
      gameList.addMouseListener(new java.awt.event.MouseAdapter() {
        public void mouseClicked(java.awt.event.MouseEvent evt) {
          gameListMouseClicked(evt);
        }
      });
      
      jPanel7.add(gameList);
      gameList.setBounds(10, 20, 150, 180);
      
      jPanel3.add(jPanel7);
      jPanel7.setBounds(530, 30, 170, 210);
      
      jPanel8.setLayout(null);
      
      jPanel8.setBorder(new javax.swing.border.TitledBorder(null, "Players", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), java.awt.Color.white));
      jPanel8.setBackground(new java.awt.Color(0, 80, 160));
      playerList.setFont(new java.awt.Font("Dialog", 1, 12));
      playerList.setEnabled(false);
      playerList.setBackground(java.awt.Color.white);
      jPanel8.add(playerList);
      playerList.setBounds(10, 20, 150, 100);
      
      jPanel3.add(jPanel8);
      jPanel8.setBounds(530, 330, 170, 130);
      
      jLabel7.setText("Last message from server:");
      jLabel7.setForeground(java.awt.Color.white);
      jLabel7.setBackground(java.awt.Color.white);
      jPanel3.add(jLabel7);
      jLabel7.setBounds(10, 580, 220, 13);
      
      messageLabel.setText("n/a");
      messageLabel.setForeground(java.awt.Color.white);
      messageLabel.setFont(new java.awt.Font("Dialog", 0, 12));
      jPanel3.add(messageLabel);
      messageLabel.setBounds(240, 580, 460, 13);
      
      jPanel3.add(gameNameEdit);
      gameNameEdit.setBounds(580, 258, 120, 17);
      
      createButton.setForeground(java.awt.Color.white);
      createButton.setText("Create new game");
      createButton.setBackground(new java.awt.Color(0, 80, 160));
      createButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          createButtonActionPerformed(evt);
        }
      });
      
      jPanel3.add(createButton);
      createButton.setBounds(540, 280, 160, 23);
      
      jPanel3.add(jSeparator1);
      jSeparator1.setBounds(10, 572, 690, 4);
      
      jLabel1.setText("Name:");
      jLabel1.setForeground(java.awt.Color.white);
      jLabel1.setFont(new java.awt.Font("Dialog", 0, 12));
      jPanel3.add(jLabel1);
      jLabel1.setBounds(530, 260, 39, 13);
      
      jLabel2.setText("Compose message:");
      jLabel2.setForeground(java.awt.Color.white);
      jLabel2.setFont(new java.awt.Font("Dialog", 0, 12));
      jPanel3.add(jLabel2);
      jLabel2.setBounds(10, 550, 122, 13);
      
      jPanel3.add(chatEdit);
      chatEdit.setBounds(150, 548, 460, 17);
      
      sendButton.setForeground(java.awt.Color.white);
      sendButton.setText("Send");
      sendButton.setBackground(new java.awt.Color(0, 80, 160));
      sendButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          sendButtonActionPerformed(evt);
        }
      });
      
      jPanel3.add(sendButton);
      sendButton.setBounds(620, 544, 80, 23);
      
      chatHistory.setBackground(java.awt.Color.white);
      jPanel3.add(chatHistory);
      chatHistory.setBounds(10, 470, 690, 70);
      
      getContentPane().add(jPanel3, java.awt.BorderLayout.CENTER);
      
      pack();
    }//GEN-END:initComponents

    private void sendButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sendButtonActionPerformed
      if (gameName != null)
        sendAndReceive(gameType + " chat " + chatEdit.getText());
      chatEdit.setText("");
    }//GEN-LAST:event_sendButtonActionPerformed

    private void gameListMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_gameListMouseClicked
      if (gameName != null) {
        Messenger m = new Messenger(this, "Please leave the current game first.", true);
        m.show();
      }
      else {
        gameName = gameList.getSelectedItem();
        if (gameName != null) {
          if (gameName.indexOf(" (") > 0)
            gameName = gameName.substring(0, gameName.indexOf(" ("));
          String response = commandSuccessful(gameType + " join " + gameName, Msg.messageString(Msg.OK_JOINED));
          if (response != null)  {
            enterGame();
            updatePlayerList();
          }
          else
            leaveGame();
        }
      }
    }//GEN-LAST:event_gameListMouseClicked

    private void gameTypeListMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_gameTypeListMouseClicked
      gameName = null;
      gameType = gameTypeList.getSelectedItem();
      if (gameType != null) {
        gameList.setEnabled(true);
        updateGameList();
        setupBoard(gameType);
      }
      else {
        if (gameList.getItemCount() > 0)
          gameList.removeAll();
        if (playerList.getItemCount() > 0)
          playerList.removeAll();
      }
    }//GEN-LAST:event_gameTypeListMouseClicked

    private void moveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveButtonActionPerformed
      String response = commandSuccessful(gameType + " move " + moveEdit.getText(),
                          Msg.messageString(Msg.OK_LEGAL_MOVE));
      if (response == null) {
        Messenger m = new Messenger(this, "Illegal move.", true);
        m.show();
      }
    }//GEN-LAST:event_moveButtonActionPerformed

    public void doMove(String move) {
      moveEdit.setText(move);
      moveButtonActionPerformed(null);
    }
    
    private void enterGame() {
      gameTypeList.setEnabled(false);
      createButton.setEnabled(false);
      gameNameEdit.setEnabled(false);
      gameList.setEnabled(false);
      gamePanel.setEnabled(true);
      canStart.setSelected(false);
      canStartValue = false;
      playerList.setEnabled(true);
      if (moveHistory.getItemCount() > 0)
        moveHistory.removeAll();
      moveCount = 0;
    }
    
    private void leaveGame() {
      gameTypeList.setEnabled(true);
      createButton.setEnabled(true);
      gameNameEdit.setEnabled(true);
      gameList.setEnabled(true);
      gamePanel.setEnabled(false);
      gameName = null;
      if (playerList.getItemCount() > 0)
        playerList.removeAll();
      updateGameList();
    }
    
    private void createButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createButtonActionPerformed
      if (gameType != null) {
        String response = commandSuccessful(gameType + " offer " + gameNameEdit.getText(),
                            Msg.messageString(Msg.OK_CREATED));
        if (response != null) {
          gameName = gameNameEdit.getText();
          updateGameList();
          updatePlayerList();
          enterGame();
        }
        else
          enterGame();
        gameNameEdit.setText("");
      }
    }//GEN-LAST:event_createButtonActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
      if (gameName != null) {
        sendAndReceive(gameType + " quit " + gameName);
        updateGameList();
        leaveGame();
      }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void canStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_canStartActionPerformed
      // Add your handling code here:
      if (gameName != null) {
        canStartValue = (!canStartValue);
        if (canStartValue)
          sendAndReceive(gameType + " canstart=true");
        else
          sendAndReceive(gameType + " canstart=false");
      }
    }//GEN-LAST:event_canStartActionPerformed

    private void updatePlayerList() {
      String response = commandSuccessful(gameType + " playerlist", Msg.messageString(Msg.OK_PLAYER_LIST));
      if (response != null)
        fillList(playerList, response);
    }
    
    private void updateGameList() {
      String response = commandSuccessful(gameType + " gamelist", Msg.messageString(Msg.OK_GAME_LIST));
      if (gameList.getItemCount() > 0)
        gameList.removeAll();
      if (response != null) {
        int index = response.indexOf(',');
        while (index > 0) {
          String game = response.substring(0, index);
          response = response.substring(index + 1);
          int bracket = game.indexOf('(');
          if (bracket > 0)
            game = game.substring(0, bracket) + " " + game.substring(bracket);
          gameList.add(game);
          index = response.indexOf(',');
        }
        if (response.length() > 1) {
          int bracket = response.indexOf('(');
          if (bracket > 0)
            response = response.substring(0, bracket) + " " + response.substring(bracket);
          gameList.add(response.substring(0, response.length() - 1));
        }
      }
    }

    private void send(String message) {
      int errorCnt = 0;
      message += "\n";
      if (message == null)
        return;
      do {
        try {
          out.write(message.getBytes());
          out.flush();
          errorCnt = 10;
        }
        catch (Exception e) {
          try { Thread.sleep(100); }
            catch (InterruptedException e2) { }
        }
      } while (errorCnt++ < 9);
    }
    
    private void receiveAll() {
      String received = updater.getBuffer();
      int index = received.indexOf('\n');
      while (index >= 0) {
        String dummy = decrypt(received.substring(0, index));
        received = received.substring(index + 1);
        if (dummy.compareTo("ping") != 0) {
          if (dummy.startsWith("[NEWS] "))
            processNews(dummy.substring(7));
          else
            messageLabel.setText(dummy);
        }
        index = received.indexOf('\n');
      }
      updater.insertBuffer(received);
    }

    private String receiveOk() {
      int errorCnt = 0;
      String received = "";
      do {
        received = received + updater.getBuffer();
        int index = received.indexOf('\n');
        while (index >= 0) {
          String dummy = received.substring(0, index);
          received = received.substring(index + 1);
          if (dummy.compareTo("ping") != 0) {
            dummy = decrypt(dummy);
            messageLabel.setText(dummy);
            if ((dummy.startsWith("[OK ")) || (dummy.startsWith("[ERROR "))) {
              updater.insertBuffer(received);
              return dummy;
            }
          }
          index = received.indexOf('\n');
        }
        try { Thread.sleep(100); }
          catch (InterruptedException e) { }
      } while (errorCnt++ < 20);
      updater.insertBuffer(received);
      return null;
    }

    private String commandSuccessful(String message, String header) {
      String result = sendAndReceive(message);
      if (result != null) {
        if ((result.length() >= header.length()) &&
            (result.substring(0, header.length()).compareTo(header) == 0))
          return result.substring(header.length());
        else
          return null;
      }
      return null;
    }
    
    private String sendAndReceive(String message) {
      if (message == null)
        return null;
      if (newsAgent != null) {
        newsAgent.active = false;
        try { Thread.sleep(10); }
         catch (InterruptedException e) { }
        receiveAll();
        if (remoteCryptor != null)
          message = remoteCryptor.encrypt(localCryptor.sign(message));
        send(message);
        String result = receiveOk();
        newsAgent.active = true;
        return result;
      }
      else
        return null;
    }

    private String sendAndReceivePlain(String message) {
      if (message == null)
        return null;
      receiveAll();
      send(message);
      String result = receiveOk();
      return result;
    }
    
    public String decrypt(String msg) {
      if (remoteCryptor == null)
        return msg;
      else {
        String result = remoteCryptor.verify(localCryptor.decrypt(msg));
        if (result == null)
          return "(Could not decrypt message.)";
        else
          return result;
      }
    }

    private void fillList(java.awt.List list, String elements) {
      if (list.getItemCount() > 0)
        list.removeAll();
      if (elements != null) {
        int index = elements.indexOf(',');
        while (index > 0) {
          String element = elements.substring(0, index);
          elements = elements.substring(index + 1);
          list.add(element);
          index = elements.indexOf(',');
        }
        if (elements.length() > 1)
          list.add(elements.substring(0, elements.length() - 1));
      }
    }
    
    private void connect() {
      gameTypeList.setEnabled(true);
      gameType = null;
      gameName = null;
      connectButton.setText("Disconnect");
      connectionState.setText("(connected)");
      if (gameList.getItemCount() > 0)
        gameList.removeAll();
      if (playerList.getItemCount() > 0)
        playerList.removeAll();
      String gameList = commandSuccessful("gamelist", Msg.messageString(Msg.OK_GAME_LIST));
      fillList(gameTypeList, gameList);
      playerName = playerEdit.getText();
      chatEdit.setEnabled(true);
      sendButton.setEnabled(true);
    }

    private void disconnect() {
      if (connection != null)
        if ((playerName != null) && (playerName.length() > 0))
          sendAndReceive("deregister " + playerName);

      updater.terminated = true;
      newsAgent.terminated = true;
      try {
        if (in != null) in.close();
        if (out != null) out.close();
      }
      catch (IOException io) {
      }
      in = null;
      out = null;
      newsAgent = null;
      updater = null;
      connection = null;
      connected = false;
      connectButton.setText("Connect");
      connectionState.setText("(unconnected)");

      if (gameTypeList.getItemCount() > 0)
        gameTypeList.removeAll();
      if (gameList.getItemCount() > 0)
        gameList.removeAll();
      if (playerList.getItemCount() > 0)
        playerList.removeAll();

      leaveGame();
      gameTypeList.setEnabled(false);
      chatEdit.setEnabled(false);
      sendButton.setEnabled(false);
      
      remoteCryptor = null;
      gameType = null;
      gameName = null;
      setupBoard("");

      try {
        Thread.sleep(50);
      } catch (Exception e666) { }

      updater = new UpdateThread(this);
      updater.start();
      newsAgent = new NewsAgent(this, updater);
      newsAgent.start();
    }

    private boolean setupSecureCommunication() {
      Cryptor tempCryptor;
      remoteCryptor = null;
      String remoteKey = sendAndReceivePlain("getkey");
      String header = Msg.messageString(Msg.OK_PUBKEY);
      if ((remoteKey != null) && (remoteKey.length() > header.length()) &&
          (remoteKey.substring(0, header.length()).compareTo(header) == 0)) {
        remoteKey = remoteKey.substring(header.length());
        if ((remoteKey.length() > 3) &&
            (remoteKey.substring(0, 3).compareTo("RSA") == 0)) {
              remoteCryptor = new RSA(remoteKey);
          if (remoteCryptor.getPublicKey() != null) {
            String response = sendAndReceivePlain("setkey " + localCryptor.exportPublicKey());
            if ((response != null) &&
                (response.compareTo(Msg.messageString(Msg.OK_PUBKEY_RECVD)) == 0))
              return true;
          }
        }
      }
      return false;
    }

    private void connectButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectButtonActionPerformed
      if (!connected) {
        String serverName, playerName;
        int portNumber;
        try {
          serverName = serverEdit.getText();
          playerName = playerEdit.getText();
          portNumber = Integer.parseInt(portEdit.getText());
          connection = null;
          if (serverName.length() == 0)
            throw new Exception("server");
          if (playerName.length() == 0)
            throw new Exception("player");
          try {
            connection = new Socket(serverName, portNumber);
            in = connection.getInputStream();
            out = connection.getOutputStream();
          }
          catch (Exception e2) {
            throw new Exception("connection");
          }
          connected = true;
          updater.setConnection(connection);
          if (!setupSecureCommunication()) {
            disconnect();
            throw new Exception("connection");
          }
          playerName = sendAndReceive("register " + playerName);
          if ((playerName != null) &&
              (playerName.compareTo(Msg.messageString(Msg.OK_REGISTERED)) == 0))
            connect();
          else {
            disconnect();
            throw new Exception("playername");
          }
        } catch (Exception ne) {
          if (ne.getMessage() == null) {
            System.err.println(ne.toString());
            ne.printStackTrace();
          }
          else {
            if (ne.getMessage().compareTo("server") == 0) {
              Messenger m = new Messenger(this, "Please enter the server name.", true);
              m.show();
            }
            else if (ne.getMessage().compareTo("player") == 0) {
              Messenger m = new Messenger(this, "Please enter your player name.", true);
              m.show();
            }
            else if (ne.getMessage().compareTo("connection") == 0) {
              Messenger m = new Messenger(this, "Could not connect to server.", true);
              m.show();
            }
            else if (ne.getMessage().compareTo("playername") == 0) {
              Messenger m = new Messenger(this, "Please try a different player name.", true);
              m.show();
            }
            else {
              Messenger m = new Messenger(this, "Please enter a valid port number.", true);
              m.show();
            }
          }
          disconnect();
        }
      }
      else
        disconnect();
    }//GEN-LAST:event_connectButtonActionPerformed

    public void setMove(String move) {
      moveEdit.setText(move);
    }
    
    public void processNews(String news) {
      messageLabel.setText("[NEWS] " + news);
      int len = news.length();
      if (news.startsWith("Chat from ")) {
        news = news.substring(10);
        chatHistory.add(news, 0);
        if (chatHistory.getItemCount() > 32)
          chatHistory.remove(32);
      }
      else if (news.startsWith("board=")) {
        boardState.setBoardState(news.substring(6));
        boardState.repaint();
      }
      else if (news.compareTo("Time out.") == 0) {
//        Messenger m = new Messenger(this, "Your time is out.", true);
//        m.show();
      }
      else if (news.endsWith("entered the game.")) {
        soundAgent.playFile("sound/newplayer.wav");
        updatePlayerList();
      }
      else if (news.endsWith("left the game.")) {
        updatePlayerList();
      }
      else if (news.endsWith("Countdown: 0 seconds."))
        soundAgent.playFile("sound/goodluck.wav");
      else if (news.compareTo("You have won.") == 0) {
        soundAgent.playFile("sound/winner.wav");
        Messenger m = new Messenger(this, "Congratz! You have won.", true);
        m.show();
        leaveGame();
      }
      else if (news.indexOf("has won") > 0) {
        soundAgent.playFile("sound/loser.wav");
        leaveGame();
      }
      else if (news.indexOf("has moved:") > 0) {
        int index1 = news.indexOf(": ");
        int index2 = news.indexOf(" has moved");
        if ((index1 > index2) && (index2 > 0)) {
          moveCount++;
          String move = news.substring(index1 + 2).toUpperCase();
          moveHistory.add("(" + moveCount + ") " + news.substring(0, index2) + ": " + move);
        }
      }
      else if (news.startsWith("You are "))
        boardState.setWho(news.charAt(8));
    }
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
      newsAgent.terminated = true;
      if (updater != null)
        updater.terminated = true;
      soundAgent.terminated = true;
      System.exit(0);
    }//GEN-LAST:event_exitForm

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
      GameClient gc = new GameClient();
      gc.setBounds(100, 100, 720, 640);
      gc.setLocation(100, 100);
      gc.setResizable(false);
      gc.setTitle("Java GameClient for Stu's Internet Game Server");
      gc.show();
    }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JTextField portEdit;
    private javax.swing.JLabel connectionState;
    private javax.swing.JButton connectButton;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JTextField playerEdit;
    private javax.swing.JTextField serverEdit;
    private javax.swing.JPanel gamePanel;
    private javax.swing.JCheckBox canStart;
    private javax.swing.JButton jButton1;
    private javax.swing.JTextField moveEdit;
    private javax.swing.JButton moveButton;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private java.awt.List moveHistory;
    private javax.swing.JPanel jPanel6;
    private java.awt.List gameTypeList;
    private javax.swing.JPanel jPanel7;
    private java.awt.List gameList;
    private javax.swing.JPanel jPanel8;
    private java.awt.List playerList;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel messageLabel;
    private javax.swing.JTextField gameNameEdit;
    private javax.swing.JButton createButton;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JTextField chatEdit;
    private javax.swing.JButton sendButton;
    private java.awt.List chatHistory;
    // End of variables declaration//GEN-END:variables

}


class UpdateThread extends Thread {
  private static final int INTERVAL = 40;
  
  private GameClient owner;
  public boolean terminated;
  public Socket connection;
  private boolean listen;
  private String inBuffer;
  private byte buffer[] = new byte[2048];
  private Random rand;
  
  public UpdateThread(GameClient owner) {
    this.owner = owner;
    inBuffer = "";
    terminated = false;
    connection = null;
    listen = true;
    rand = new Random();
  }
  
  public void setConnection(Socket connection) {
    inBuffer = "";
    this.connection = connection;
  }

  public String getBuffer() {
    listen = false;
    try { sleep(INTERVAL / 5); }
      catch (Exception e) { }
    String result = inBuffer;
    inBuffer = "";
    listen = true;
    return result;
  }
  
  public void insertBuffer(String text) {
    listen = false;
    try { sleep(INTERVAL / 5); }
      catch (Exception e) { }
    inBuffer = text + inBuffer;
    listen = true;
  }
  
  public void run() {
    int available;
    while (!terminated) {
      try {
        sleep(INTERVAL);
        if ((connection != null) && (listen)) {
          available = connection.getInputStream().available();
          if (available > 0) {
            if (available > 2000)
              available = 2000;
            connection.getInputStream().read(buffer, 0, available);
            inBuffer += new String(buffer, 0, available);
          }
        }
      }
      catch (Exception e) {
      }
    }
    connection = null;
  }
  
}

class NewsAgent extends Thread {
  private static final int INTERVAL = 100;
  
  private GameClient owner;
  private UpdateThread updater;
  public boolean terminated, active;
 
  public NewsAgent(GameClient owner, UpdateThread updater) {
    this.owner = owner;
    this.updater = updater;
    terminated = false;
    active = true;
  }
  
  public void run() {
    while (!terminated) {
      try { sleep(INTERVAL); }
        catch (InterruptedException e) { };
      if (active) {
        String buffer = updater.getBuffer();
        int index = buffer.indexOf('\n');
        if (index > 3) {
          String message = buffer.substring(0, index);
          updater.insertBuffer(buffer.substring(index + 1));
          if (message.compareTo("ping") != 0) {
            String decrypted = owner.decrypt(message);
            if ((decrypted != null) && (decrypted.startsWith("[NEWS] ")))
              owner.processNews(decrypted.substring(7));
            else
              updater.insertBuffer(message + "\n");
          }
        }
      }
    }
    updater = null;
  }
  
}

class SoundAgent extends Thread {
  private String buffer;
  public  boolean terminated;
  private boolean playing;

  public SoundAgent() {
    buffer = "";
    terminated = false;
    playing = false;
  }

  public void playFile(String fileName) {
    playing = false;
    if (fileName != null)
      buffer = fileName;
  }

  public void run() {
    AudioClip clip = null;
    URL url = null;
    while (!terminated) {
      try {
        sleep(100);
        if (buffer.length() > 0) {
          String fileName = buffer;
          buffer = "";
          url = new URL("file", "", fileName);
          clip = Applet.newAudioClip(url);
          clip.play();
          playing = true;
          int cnt = 0;
          while ((!terminated) && (playing) && (cnt++ < 50))
            sleep(100);
          if (clip != null)
            clip.stop();
          clip = null;
          url = null;
        }
      }
      catch (Exception e) {
        if (clip != null)
          clip.stop();
        clip = null;
        url = null;
      }
    }
  }

}
