/********************************************************************************
 * GameServer.java
 * Copyright (c) 2002 by Stefan Buettcher
 *
 * This class is the boss of all games currently running and is used for
 * passing messages from the ConnectionMaster to the respective Games and
 * vice versa
 *
 *******************************************************************************/

import java.net.*;
import java.util.*;

public class GameServer {
  private ConnectionMaster connMaster;
  private UpdateAgent updateAgent;
  private int portNumber;

  Vector games, players;

  public GameServer(int port) {
    games = new Vector();
    players = new Vector();
    portNumber = port;
  }

  public void addGame(Game aGame) {
    games.addElement(aGame);
  }
  
  public void addPlayer(Player aPlayer) {
    players.addElement(aPlayer);
  }
  
  public void removePlayer(Player aPlayer) {
    players.remove(aPlayer);
    if (aPlayer.getMatch() != null)
      aPlayer.getMatch().removePlayer(aPlayer);
  }
  
  public void processMessage(Player p, String message) {
    if (message.compareTo("gamelist") == 0) {
      if (games.size() == 0)
        p.sendMessage(Msg.messageString(Msg.OK_NO_GAMES));
      else {
        String result = Msg.messageString(Msg.OK_GAME_LIST);
        for (int i = 0; i < games.size() - 1; i++)
          result += ((Game)games.elementAt(i)).getName() + ",";
        result += ((Game)games.elementAt(games.size() - 1)).getName() + ".";
        p.sendMessage(result);
      }
      return;
    }
    int index = message.indexOf(' ');
    if (index > 0) {
      String gameName = message.substring(0, index);
      message = message.substring(index + 1);
      for (int i = 0; i < games.size(); i++)
        if (((Game)games.elementAt(i)).getName().compareTo(gameName) == 0) {
          ((Game)games.elementAt(i)).processMessage(p, message);
          return;
        }
      p.sendMessage("[ERROR] The game you specified does not exist.");
    }
    else
      p.sendMessage("[ERROR] Invalid message format.");
  }

  public void doSomeWork() {
    for (int i = 0; i < games.size(); i++)
      try { ((Game)games.elementAt(i)).doSomeWork(); }
        catch (Exception e) { }
  }

  public void run() {
    updateAgent = new UpdateAgent(this);
    connMaster = new ConnectionMaster(this, portNumber);
    updateAgent.start();
  }

  public static void main(String args[]) {
    int port = 0;
    if (args.length == 1) {
      try {
        port = Integer.parseInt(args[0]);
        GameServer server = new GameServer(port);
        server.addGame(new Game("4Gewinnt", server));
        server.addGame(new Game("Checkers", server));
        server.addGame(new Game("Chess", server));
        server.run();
        try {
          URL url = new URL("http", "stefan.buettcher.org", "/cs/gameserver/addserver.php");
          url.openStream();
          System.out.println("GameServer database updated.");
        }
        catch (Exception e) {
          System.out.println("Could not connect to GameServer database.");
          System.out.println("(This does not affect program functionality.)");
        }
      }
      catch (NumberFormatException e) {
        System.out.println("Error! Please enter a valid port number.");
      }
    }
    else
      System.out.println("Correct Syntax: java GameServer PORTNUMBER\n");
  }
  
}


class UpdateAgent extends Thread {
  private static final int INTERVAL = 100;

  private GameServer owner;
  public boolean terminated = false;

  public UpdateAgent(GameServer anOwner) {
    owner = anOwner;
  }

  public void run() {
    while (!terminated) {
      try {
        sleep(INTERVAL);
      }
      catch (InterruptedException irq) {
      }
      owner.doSomeWork();
    }
  }
}

