/********************************************************************************
 * Messenger.java
 * Copyright (c) 2002 by Stefan Buettcher
 *
 * The Messenger class performs simple Dialog operations, i.e. notifying the
 * user of an important event, e.g. time-out.
 *
 *******************************************************************************/

import java.awt.*;

public class Messenger extends javax.swing.JDialog {
    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /** Creates new form Messenger */
    public Messenger(java.awt.Frame parent, String message, boolean modal) {
      super(parent, modal);
      initComponents();
      messageLabel.setText("  " + message);
      this.setSize(320, this.getHeight());
      this.setTitle("Message");
      this.setLocation(parent.getX() + 100, parent.getY() + 80);
    }

    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
      return returnStatus;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
      buttonPanel = new javax.swing.JPanel();
      okButton = new javax.swing.JButton();
      messageLabel = new javax.swing.JLabel();
      jLabel2 = new javax.swing.JLabel();
      
      addWindowListener(new java.awt.event.WindowAdapter() {
        public void windowClosing(java.awt.event.WindowEvent evt) {
          closeDialog(evt);
        }
      });
      
      buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
      
      okButton.setText("OK");
      okButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
          okButtonActionPerformed(evt);
        }
      });
      
      buttonPanel.add(okButton);
      
      getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);
      
      messageLabel.setText(" ");
      messageLabel.setForeground(java.awt.Color.black);
      getContentPane().add(messageLabel, java.awt.BorderLayout.CENTER);
      
      jLabel2.setText(" ");
      getContentPane().add(jLabel2, java.awt.BorderLayout.NORTH);
      
      pack();
    }//GEN-END:initComponents

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
      doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
      doClose(RET_CANCEL);
    }//GEN-LAST:event_closeDialog

    private void doClose(int retStatus) {
      returnStatus = retStatus;
      setVisible(false);
      dispose();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel messageLabel;
    private javax.swing.JLabel jLabel2;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;
}
