/********************************************************************************
 * Messages.java
 * Copyright (c) 2002 by Stefan Buettcher
 *
 * This class performs the conversion of (HTTP-like) response codes to
 * Message strings.
 *
 *******************************************************************************/

public class Msg {
  public static final int SELECTION_BITS    = 0x700;
  public static final int OK                = 0x100;
  public static final int OK_CREATED        = OK + 1;      // upon a successful "offer" command
  public static final int OK_JOINED         = OK + 2;      // upon a successful "join" command
  public static final int OK_REGISTERED     = OK + 3;      // upon a successful "register" command
  public static final int OK_DEREGISTERED   = OK + 4;      // upon a successful "deregister" command
  public static final int OK_PUBKEY_RECVD   = OK + 5;      // upon a successful "setkey=..." command
  public static final int OK_PUBKEY         = OK + 6;      // upon a successful "getkey" command
  public static final int OK_LEFT_GAME      = OK + 7;      // upon a successful "quit" command
  public static final int OK_NO_GAMES       = OK + 8;      // upon a successful "gamelist" command
  public static final int OK_GAME_LIST      = OK + 9;      // upon a successful "gamelist" command
  public static final int OK_READY          = OK + 10;     // upon a successful "canstart=true" command
  public static final int OK_NOT_READY      = OK + 11;     // upon a successful "canstart=false" command
  public static final int OK_BOARD          = OK + 12;     // upon a successful "getboard" command
  public static final int OK_HAS_TURN       = OK + 13;     // upon a successful "whoseturn" command
  public static final int OK_TIME_LEFT      = OK + 14;     // upon a successful "timeleft" command
  public static final int OK_LEGAL_MOVE     = OK + 15;     // upon a successful "move" command
  public static final int OK_PLAYER_LIST    = OK + 16;     // upon a successful "playerlist" command
  public static final int ERR               = 0x200;
  public static final int ERR_UNREGISTERED  = ERR + 1;     // means: "please register first"
  public static final int ERR_DUPL_PLAYER   = ERR + 2;     // means: "please choose a different name"
  public static final int ERR_ONLY_ONE_ID   = ERR + 3;     // means: "please register only once"
  public static final int ERR_ILLEGAL_FORMAT= ERR + 4;     // means: "i don't understand you"
  public static final int ERR_INTERNAL_ERROR= ERR + 5;     // means: "i am buggy"
  public static final int ERR_DUPL_GAME     = ERR + 6;     // means: "please choose a different game name"
  public static final int ERR_IN_GAME       = ERR + 7;     // means: "please leave the current game first"
  public static final int ERR_NOT_IN_GAME   = ERR + 8;     // means: "you can't do that because you aren't in a game"
  public static final int ERR_NO_SUCH_GAME  = ERR + 9;     // means: "you cannot join a non-existing game"
  public static final int ERR_GAME_FULL     = ERR + 10;    // means: "you cannot enter the game, since it is full"
  public static final int ERR_ILLEGAL_MOVE  = ERR + 11;    // means: "this is no legal move"
  public static final int ERR_NOT_YOUR_TURN = ERR + 12;    // means: "please do not move if it is not your turn"
  public static final int NEWS              = 0x300;

  public static String messageString(int resCode) {
    String result = "[OTHER " + resCode + "]";
    if ((resCode & SELECTION_BITS) == OK) {
      result = "[OK " + resCode + "] ";
      if (resCode == OK_CREATED)
        result += "Game created.";
      else if (resCode == OK_JOINED)
        result += "Joined game.";
      else if (resCode == OK_REGISTERED)
        result += "Registered.";
      else if (resCode == OK_DEREGISTERED)
        result += "Deregistered.";
      else if (resCode == OK_PUBKEY_RECVD)
        result += "Public Key received.";
      else if (resCode == OK_PUBKEY)
        result += "Public Key: ";
      else if (resCode == OK_LEFT_GAME)
        result += "Game left.";
      else if (resCode == OK_NO_GAMES)
        result += "No games available.";
      else if (resCode == OK_GAME_LIST)
        result += "Available games: ";
      else if (resCode == OK_READY)
        result += "You are ready.";
      else if (resCode == OK_NOT_READY)
        result += "You are not ready.";
      else if (resCode == OK_BOARD)
        result += "Board: ";
      else if (resCode == OK_TIME_LEFT)
        result += "Time left: ";
      else if (resCode == OK_HAS_TURN)
        result += "Has turn: ";
      else if (resCode == OK_LEGAL_MOVE)
        result += "Legal move.";
      else if (resCode == OK_PLAYER_LIST)
        result += "Players: ";
    }
    else if ((resCode & SELECTION_BITS) == ERR) {
      result = "[ERROR " + resCode + "] ";
      if (resCode == ERR_UNREGISTERED)
        result += "Unregistered. Please register first.";
      else if (resCode == ERR_DUPL_PLAYER)
        result += "There is already a player with that name.";
      else if (resCode == ERR_ONLY_ONE_ID)
        result += "You cannot have more than one ID per connection.";
      else if (resCode == ERR_ILLEGAL_FORMAT)
        result += "Illegal message.";
      else if (resCode == ERR_INTERNAL_ERROR)
        result += "Internal error.";
      else if (resCode == ERR_IN_GAME)
        result += "Please leave the current game first.";
      else if (resCode == ERR_DUPL_GAME)
        result += "There is already a game with that name.";
      else if (resCode == ERR_NOT_YOUR_TURN)
        result += "It is not your turn, brain!";
      else if (resCode == ERR_NOT_IN_GAME)
        result += "You are no member of that game.";
      else if (resCode == ERR_NO_SUCH_GAME)
        result += "No such game.";
      else if (resCode == ERR_GAME_FULL)
        result += "There are no free places left.";
      else if (resCode == ERR_ILLEGAL_MOVE)
        result += "Illegal move.";
    }
    else if ((resCode & SELECTION_BITS) == NEWS) {
    }
    return result;
  }
  
  public Msg() {
  }

}
