import java.io.*;
import java.net.*;

public class SampleClient {
  Socket s;
  InputStream in;
  OutputStream out;
  String inBuffer;
  Cryptor localCryptor;
  Cryptor remoteCryptor = null;
  Receiver receiver;
  
  public SampleClient() {
    inBuffer = "";
    localCryptor = new RSA();
  }
  
  private void sendMessage(String message) {
    try {
      message += "\n";
      s.getOutputStream().write(message.getBytes());
      s.getOutputStream().flush();
    }
    catch (Exception e) {
      System.out.println("Exception: " + e);
    }
  }
  
  private void sendEncryptedMessage(String message) {
    sendMessage(remoteCryptor.encrypt(localCryptor.sign(message)));
  }
  
  private String decryptMessage(String message) {
    if (remoteCryptor == null)
      return message;
    else
      return remoteCryptor.verify(localCryptor.decrypt(message));
  }
  
  private String sendMessageReceiveResponse(String message) {
    while (receiver.messageCount() > 0)
      System.out.println("Received message: " +
                         decryptMessage(receiver.getMessage(0)));
    if (remoteCryptor == null)
      sendMessage(message);
    else
      sendEncryptedMessage(message);
    int cnt = 0;
    while ((receiver.messageCount() == 0) && (++cnt < 100))
      try {
        Thread.sleep(40);
      }
      catch (Exception e) {
      }
    if (cnt >= 100)
      return null;
    else {
      return decryptMessage(receiver.getMessage(0));
    }
  }
  
  public void run() {
    String input;
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    try {
      s = new Socket("localhost", 1234);
      in = s.getInputStream();
      out = s.getOutputStream();
      Thread.sleep(100);
      receiver = new Receiver(in);
      receiver.start();
      String response = sendMessageReceiveResponse("getkey");
      if (response.length() > 21)
        response = response.substring(21);
      Cryptor tempCryptor = new RSA(response);
      if (tempCryptor.getPublicKey() == null) {
        System.out.println("A problem occurred when receiving the server's public key.");
        return;
      }
      response = sendMessageReceiveResponse("setkey " + localCryptor.exportPublicKey());
      remoteCryptor = tempCryptor;
      System.out.println(decryptMessage(response));
      do {
        if (System.in.available() > 0) {
          input = br.readLine();
          System.out.println(sendMessageReceiveResponse(input));
        }
        else {
          input = "";
          while (receiver.messageCount() > 0)
            System.out.println(decryptMessage(receiver.getMessage(0)));
        }
      } while ((input.compareTo("end") != 0) && (input.compareTo("stop") != 0));
    } catch (Exception e) {
      System.out.println("Exception: " + e);
    }
    receiver.terminated = true;
    receiver = null;
  }
  
  public static void main(String args[]) {
    SampleClient sc = new SampleClient();
    sc.run();
  }

}

class Receiver extends Thread {
  private InputStream in;
  public boolean terminated;
  public String inBuffer;
  private byte buffer[];
  
  private int semaphore;
  
  public Receiver(InputStream i) {
    in = i;
    inBuffer = "";
    buffer = new byte[2048];
    semaphore = 1;
  }
  
  public int messageCount() {
    int result = 0;
    semaphoreP();
    for (int i = 0; i < inBuffer.length(); i++)
      if (inBuffer.charAt(i) == '\n')
        result++;
    semaphoreV();
    return result;
  }
  
  public String getMessage(int index) {
    int cnt = 0;
    int pos = 0;
    semaphoreP();
    do {
      if (cnt == index) {
        String s = inBuffer.substring(pos);
        cnt = s.indexOf('\n');
        if (cnt >= 0) {
          inBuffer = inBuffer.substring(0, pos) + inBuffer.substring(cnt + 1);
          semaphoreV();
          return s.substring(0, cnt);
        }
        else {
          inBuffer = inBuffer.substring(0, pos);
          semaphoreV();
          return s;
        }
      }
      if (inBuffer.charAt(pos) == '\n')
        cnt++;
      pos++;
    } while (pos < inBuffer.length());
    semaphoreV();
    return null;
  }
  
  private void semaphoreP() {
    while (semaphore <= 0)
      try { Thread.sleep(10); }
        catch (InterruptedException ie) { }
    semaphore--;
    if (semaphore < 0) {
      semaphore++;
      semaphoreP();
    }
  }
  
  private void semaphoreV() {
    semaphore++;
  }

  public void run() {
    while (!terminated) {
      try {
        sleep(30);
        int avail = in.available();
        if (avail > 2000) avail = 2000;
        if (avail > 0) {
          in.read(buffer, 0, avail);
          semaphoreP();
          inBuffer += new String(buffer, 0, avail);
          semaphoreV();
        }
      } catch (Exception e) {
      }
    }
  }
  
}